@@ -1,4 +1,7 @@
+# -*- coding: utf-8 -*-
 """
+.. _ex-tfr-erds:
+
 ===============================
 Compute and visualize ERDS maps
 ===============================
@@ -55,7 +58,7 @@ raw.rename_channels(lambda x: x.strip('.'))  # remove dots from channel names
 events, _ = mne.events_from_annotations(raw, event_id=dict(T1=2, T2=3))
 
 # %%
-# Now we can create 5s epochs around events of interest.
+# Now we can create 5-second epochs around events of interest.
 tmin, tmax = -1, 4
 event_ids = dict(hands=2, feet=3)  # map event IDs to tasks
 
@@ -66,8 +69,9 @@ epochs = mne.Epochs(raw, events, event_ids, tmin - 0.5, tmax + 0.5,
 # Here we set suitable values for computing ERDS maps.
 freqs = np.arange(2, 36)  # frequencies from 2-35Hz
 vmin, vmax = -1, 1.5  # set min and max ERDS values in plot
-baseline = [-1, 0]  # baseline interval (in s)
-cnorm = TwoSlopeNorm(vmin=-1, vcenter=0, vmax=1.5)  # min, center, and max ERDS
+baseline = (-1, 0)  # baseline interval (in s)
+cnorm = TwoSlopeNorm(vmin=vmin, vcenter=0, vmax=vmax)  # min, center & max ERDS
+
 kwargs = dict(n_permutations=100, step_down_p=0.05, seed=1,
               buffer_size=None, out_type='mask')  # for cluster test
 
@@ -105,7 +109,7 @@ for event in event_ids:
         if ch != 0:
             ax.set_ylabel("")
             ax.set_yticklabels("")
-    fig.colorbar(axes[0].images[-1], cax=axes[-1])
+    fig.colorbar(axes[0].images[-1], cax=axes[-1]).ax.set_yscale("linear")
     fig.suptitle(f"ERDS ({event})")
     plt.show()
 
@@ -172,8 +176,7 @@ g = sns.FacetGrid(df_mean, col='condition', col_order=['hands', 'feet'],
 g = (g.map(sns.violinplot, 'channel', 'value', 'band', n_boot=10,
            palette='deep', order=['C3', 'Cz', 'C4'],
            hue_order=freq_bands_of_interest,
-           linewidth=0.5)
-      .add_legend(ncol=4, loc='lower center'))
+           linewidth=0.5).add_legend(ncol=4, loc='lower center'))
 
 g.map(plt.axhline, **axline_kw)
 g.set_axis_labels("", "ERDS (%)")
