@@ -6,24 +6,26 @@ import numpy as np
 from scipy import linalg
 from scipy.spatial.distance import cdist
 from numpy.testing import (assert_array_almost_equal, assert_array_equal,
-                           assert_almost_equal, assert_allclose,
-                           assert_array_less)
+                           assert_allclose, assert_array_less)
 
 import mne
+from mne.transforms import apply_trans, invert_transform
 from mne import (convert_forward_solution, read_forward_solution, compute_rank,
                  VolVectorSourceEstimate, VolSourceEstimate, EvokedArray,
-                 pick_channels_cov)
+                 pick_channels_cov, read_vectorview_selection)
 from mne.beamformer import (make_lcmv, apply_lcmv, apply_lcmv_epochs,
-                            apply_lcmv_raw, tf_lcmv, Beamformer,
+                            apply_lcmv_raw, Beamformer,
                             read_beamformer, apply_lcmv_cov, make_dics)
 from mne.beamformer._compute_beamformer import _prepare_beamformer_input
-from mne.beamformer._lcmv import _lcmv_source_power
 from mne.datasets import testing
 from mne.fixes import _get_args
 from mne.io.compensator import set_current_comp
+from mne.io.constants import FIFF
 from mne.minimum_norm import make_inverse_operator, apply_inverse
+from mne.minimum_norm.tests.test_inverse import _assert_free_ori_match
 from mne.simulation import simulate_evoked
-from mne.utils import object_diff, requires_h5py, catch_logging
+from mne.utils import (object_diff, requires_version, catch_logging,
+                       _record_warnings)
 
 
 data_path = testing.data_path(download=False)
@@ -36,6 +38,7 @@ fname_fwd_vol = op.join(data_path, 'MEG', 'sample',
 fname_event = op.join(data_path, 'MEG', 'sample',
                       'sample_audvis_trunc_raw-eve.fif')
 fname_label = op.join(data_path, 'MEG', 'sample', 'labels', 'Aud-lh.label')
+ctf_fname = op.join(data_path, 'CTF', 'somMDYO-18av.ds')
 
 reject = dict(grad=4000e-13, mag=4e-12)
 
@@ -69,7 +72,7 @@ def _get_data(tmin=-0.1, tmax=0.15, all_forward=True, epochs=True,
     # Setup for reading the raw data
     raw.info['bads'] = ['MEG 2443', 'EEG 053']  # 2 bad channels
     # Set up pick list: MEG - bad channels
-    left_temporal_channels = mne.read_selection('Left-temporal')
+    left_temporal_channels = read_vectorview_selection('Left-temporal')
     picks = mne.pick_types(raw.info, meg=True,
                            selection=left_temporal_channels)
     picks = picks[::2]  # decimate for speed
@@ -94,8 +97,7 @@ def _get_data(tmin=-0.1, tmax=0.15, all_forward=True, epochs=True,
             baseline=(None, 0), preload=epochs_preload, reject=reject)
         if epochs_preload:
             epochs.resample(200, npad=0)
-        with pytest.warns(RuntimeWarning, match='baseline = None'):
-            epochs.crop(0, None)
+        epochs.crop(0, None)
         evoked = epochs.average()
         info = evoked.info
     else:
@@ -117,6 +119,7 @@ def _get_data(tmin=-0.1, tmax=0.15, all_forward=True, epochs=True,
         forward_surf_ori, forward_fixed, forward_vol
 
 
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_lcmv_vector():
     """Test vector LCMV solutions."""
@@ -125,13 +128,14 @@ def test_lcmv_vector():
     # For speed and for rank-deficiency calculation simplicity,
     # just use grads
     info = mne.pick_info(info, mne.pick_types(info, meg='grad', exclude=()))
-    info.update(bads=[], projs=[])
+    with info._unlock():
+        info.update(bads=[], projs=[])
 
     forward = mne.read_forward_solution(fname_fwd)
     forward = mne.pick_channels_forward(forward, info['ch_names'])
-    vertices = [s['vertno'][::100] for s in forward['src']]
+    vertices = [s['vertno'][::200] for s in forward['src']]
     n_vertices = sum(len(v) for v in vertices)
-    assert 5 < n_vertices < 20
+    assert n_vertices == 4
 
     amplitude = 100e-9
     stc = mne.SourceEstimate(amplitude * np.eye(n_vertices), vertices,
@@ -204,94 +208,91 @@ def test_lcmv_vector():
 
 
 @pytest.mark.slowtest
-@requires_h5py
+@requires_version('h5io')
 @testing.requires_testing_data
-@pytest.mark.parametrize('reg', (0.01, 0.))
-@pytest.mark.parametrize('proj', (True, False))
-def test_make_lcmv(tmpdir, reg, proj):
+@pytest.mark.parametrize('reg, proj, kind', [
+    (0.01, True, 'volume'),
+    (0., False, 'volume'),
+    (0.01, False, 'surface'),
+    (0., True, 'surface'),
+])
+def test_make_lcmv_bem(tmp_path, reg, proj, kind):
     """Test LCMV with evoked data and single trials."""
     raw, epochs, evoked, data_cov, noise_cov, label, forward,\
         forward_surf_ori, forward_fixed, forward_vol = _get_data(proj=proj)
 
-    for fwd in [forward, forward_vol]:
-        filters = make_lcmv(evoked.info, fwd, data_cov, reg=reg,
-                            noise_cov=noise_cov)
-        stc = apply_lcmv(evoked, filters, max_ori_out='signed')
-        stc.crop(0.02, None)
+    if kind == 'surface':
+        fwd = forward
+    else:
+        fwd = forward_vol
+        assert kind == 'volume'
 
-        stc_pow = np.sum(np.abs(stc.data), axis=1)
+    filters = make_lcmv(evoked.info, fwd, data_cov, reg=reg,
+                        noise_cov=noise_cov)
+    stc = apply_lcmv(evoked, filters)
+    stc.crop(0.02, None)
+
+    stc_pow = np.sum(np.abs(stc.data), axis=1)
+    idx = np.argmax(stc_pow)
+    max_stc = stc.data[idx]
+    tmax = stc.times[np.argmax(max_stc)]
+
+    assert 0.08 < tmax < 0.15, tmax
+    assert 0.9 < np.max(max_stc) < 3.5, np.max(max_stc)
+
+    if kind == 'surface':
+        # Test picking normal orientation (surface source space only).
+        filters = make_lcmv(evoked.info, forward_surf_ori, data_cov,
+                            reg=reg, noise_cov=noise_cov,
+                            pick_ori='normal', weight_norm=None)
+        stc_normal = apply_lcmv(evoked, filters)
+        stc_normal.crop(0.02, None)
+
+        stc_pow = np.sum(np.abs(stc_normal.data), axis=1)
         idx = np.argmax(stc_pow)
-        max_stc = stc.data[idx]
-        tmax = stc.times[np.argmax(max_stc)]
+        max_stc = stc_normal.data[idx]
+        tmax = stc_normal.times[np.argmax(max_stc)]
 
-        assert 0.08 < tmax < 0.15, tmax
-        assert 0.9 < np.max(max_stc) < 3.5, np.max(max_stc)
+        lower = 0.04 if proj else 0.025
+        assert lower < tmax < 0.14, tmax
+        lower = 3e-7 if proj else 2e-7
+        assert lower < np.max(max_stc) < 3e-6, np.max(max_stc)
 
-        if fwd is forward:
-            # Test picking normal orientation (surface source space only).
-            filters = make_lcmv(evoked.info, forward_surf_ori, data_cov,
-                                reg=reg, noise_cov=noise_cov,
-                                pick_ori='normal', weight_norm=None)
-            stc_normal = apply_lcmv(evoked, filters, max_ori_out='signed')
-            stc_normal.crop(0.02, None)
+        # No weight normalization was applied, so the amplitude of normal
+        # orientation results should always be smaller than free
+        # orientation results.
+        assert (np.abs(stc_normal.data) <= stc.data).all()
 
-            stc_pow = np.sum(np.abs(stc_normal.data), axis=1)
-            idx = np.argmax(stc_pow)
-            max_stc = stc_normal.data[idx]
-            tmax = stc_normal.times[np.argmax(max_stc)]
+    # Test picking source orientation maximizing output source power
+    filters = make_lcmv(evoked.info, fwd, data_cov, reg=reg,
+                        noise_cov=noise_cov, pick_ori='max-power')
+    stc_max_power = apply_lcmv(evoked, filters)
+    stc_max_power.crop(0.02, None)
+    stc_pow = np.sum(np.abs(stc_max_power.data), axis=1)
+    idx = np.argmax(stc_pow)
+    max_stc = np.abs(stc_max_power.data[idx])
+    tmax = stc.times[np.argmax(max_stc)]
 
-            lower = 0.04 if proj else 0.025
-            assert lower < tmax < 0.14, tmax
-            lower = 3e-7 if proj else 2e-7
-            assert lower < np.max(max_stc) < 3e-6, np.max(max_stc)
+    lower = 0.08 if proj else 0.04
+    assert lower < tmax < 0.15, tmax
+    assert 0.8 < np.max(max_stc) < 3., np.max(max_stc)
 
-            # No weight normalization was applied, so the amplitude of normal
-            # orientation results should always be smaller than free
-            # orientation results.
-            assert (np.abs(stc_normal.data) <= stc.data).all()
+    stc_max_power.data[:, :] = np.abs(stc_max_power.data)
 
-        # Test picking source orientation maximizing output source power
-        filters = make_lcmv(evoked.info, fwd, data_cov, reg=reg,
-                            noise_cov=noise_cov, pick_ori='max-power')
-        stc_max_power = apply_lcmv(evoked, filters, max_ori_out='signed')
-        stc_max_power.crop(0.02, None)
-        stc_pow = np.sum(np.abs(stc_max_power.data), axis=1)
-        idx = np.argmax(stc_pow)
-        max_stc = np.abs(stc_max_power.data[idx])
-        tmax = stc.times[np.argmax(max_stc)]
-
-        lower = 0.08 if proj else 0.04
-        assert lower < tmax < 0.15, tmax
-        assert 0.8 < np.max(max_stc) < 3., np.max(max_stc)
-
-        stc_max_power.data[:, :] = np.abs(stc_max_power.data)
-
-        if fwd is forward:
-            # Maximum output source power orientation results should be
-            # similar to free orientation results in areas with channel
-            # coverage
-            label = mne.read_label(fname_label)
-            mean_stc = stc.extract_label_time_course(label, fwd['src'],
-                                                     mode='mean')
-            mean_stc_max_pow = \
-                stc_max_power.extract_label_time_course(label, fwd['src'],
-                                                        mode='mean')
-            assert_array_less(np.abs(mean_stc - mean_stc_max_pow), 1.0)
-
-        # Test NAI weight normalization:
-        filters = make_lcmv(evoked.info, fwd, data_cov, reg=reg,
-                            noise_cov=noise_cov, pick_ori='max-power',
-                            weight_norm='nai')
-        stc_nai = apply_lcmv(evoked, filters, max_ori_out='signed')
-        stc_nai.crop(0.02, None)
-
-        # Test whether unit-noise-gain solution is a scaled version of NAI
-        pearsoncorr = np.corrcoef(np.concatenate(np.abs(stc_nai.data)),
-                                  np.concatenate(stc_max_power.data))
-        assert_almost_equal(pearsoncorr[0, 1], 1.)
+    if kind == 'surface':
+        # Maximum output source power orientation results should be
+        # similar to free orientation results in areas with channel
+        # coverage
+        label = mne.read_label(fname_label)
+        mean_stc = stc.extract_label_time_course(
+            label, fwd['src'], mode='mean')
+        mean_stc_max_pow = \
+            stc_max_power.extract_label_time_course(
+                label, fwd['src'], mode='mean')
+        assert_array_less(np.abs(mean_stc - mean_stc_max_pow), 1.0)
 
     # Test if spatial filter contains src_type
-    assert 'src_type' in filters
+    assert filters['src_type'] == kind
 
     # __repr__
     assert len(evoked.ch_names) == 22
@@ -300,12 +301,12 @@ def test_make_lcmv(tmpdir, reg, proj):
     rank = 17 if proj else 20
     assert 'LCMV' in repr(filters)
     assert 'unknown subject' not in repr(filters)
-    assert '4157 vert' in repr(filters)
+    assert f'{fwd["nsource"]} vert' in repr(filters)
     assert '20 ch' in repr(filters)
     assert 'rank %s' % rank in repr(filters)
 
     # I/O
-    fname = op.join(str(tmpdir), 'filters.h5')
+    fname = op.join(str(tmp_path), 'filters.h5')
     with pytest.warns(RuntimeWarning, match='-lcmv.h5'):
         filters.save(fname)
     filters_read = read_beamformer(fname)
@@ -316,6 +317,9 @@ def test_make_lcmv(tmpdir, reg, proj):
     filters['rank'] = int(filters['rank'])
     assert object_diff(filters, filters_read) == ''
 
+    if kind != 'surface':
+        return
+
     # Test if fixed forward operator is detected when picking normal or
     # max-power orientation
     pytest.raises(ValueError, make_lcmv, evoked.info, forward_fixed, data_cov,
@@ -344,8 +348,6 @@ def test_make_lcmv(tmpdir, reg, proj):
     evoked_ch.pick_channels(evoked_ch.ch_names[1:])
     filters = make_lcmv(evoked.info, forward_vol, data_cov, reg=0.01,
                         noise_cov=noise_cov)
-    pytest.raises(ValueError, apply_lcmv, evoked_ch, filters,
-                  max_ori_out='signed')
 
     # Test if discrepancies in channel selection of data and fwd model are
     # handled correctly in apply_lcmv
@@ -356,20 +358,24 @@ def test_make_lcmv(tmpdir, reg, proj):
     # this channel from the data
     # also test here that no warnings are thrown - implemented to check whether
     # src should not be None warning occurs
-    with pytest.warns(None) as w:
-        stc = apply_lcmv(evoked, filters, max_ori_out='signed')
-    assert len(w) == 0
+    stc = apply_lcmv(evoked, filters)
+
     # the result should be equal to applying this filter to a dataset without
     # this channel:
-    stc_ch = apply_lcmv(evoked_ch, filters, max_ori_out='signed')
+    stc_ch = apply_lcmv(evoked_ch, filters)
     assert_array_almost_equal(stc.data, stc_ch.data)
 
     # Test if non-matching SSP projection is detected in application of filter
     if proj:
-        raw_proj = deepcopy(raw)
-        raw_proj.del_proj()
+        raw_proj = raw.copy().del_proj()
         with pytest.raises(ValueError, match='do not match the projections'):
-            apply_lcmv_raw(raw_proj, filters, max_ori_out='signed')
+            apply_lcmv_raw(raw_proj, filters)
+
+    # Test apply_lcmv_raw
+    use_raw = raw.copy().crop(0, 1)
+    stc = apply_lcmv_raw(use_raw, filters)
+    assert_allclose(stc.times, use_raw.times)
+    assert_array_equal(stc.vertices[0], forward_vol['src'][0]['vertno'])
 
     # Test if spatial filter contains src_type
     assert 'src_type' in filters
@@ -378,15 +384,14 @@ def test_make_lcmv(tmpdir, reg, proj):
     del filters['src_type']  # emulate 0.16 behaviour to cause warning
     with pytest.warns(RuntimeWarning, match='spatial filter does not contain '
                       'src_type'):
-        apply_lcmv(evoked, filters, max_ori_out='signed')
+        apply_lcmv(evoked, filters)
 
     # Now test single trial using fixed orientation forward solution
     # so we can compare it to the evoked solution
     filters = make_lcmv(epochs.info, forward_fixed, data_cov, reg=0.01,
                         noise_cov=noise_cov)
-    stcs = apply_lcmv_epochs(epochs, filters, max_ori_out='signed')
-    stcs_ = apply_lcmv_epochs(epochs, filters, return_generator=True,
-                              max_ori_out='signed')
+    stcs = apply_lcmv_epochs(epochs, filters)
+    stcs_ = apply_lcmv_epochs(epochs, filters, return_generator=True)
     assert_array_equal(stcs[0].data, next(stcs_).data)
 
     epochs.drop_bad()
@@ -401,14 +406,14 @@ def test_make_lcmv(tmpdir, reg, proj):
     # compare it to the solution using evoked with fixed orientation
     filters = make_lcmv(evoked.info, forward_fixed, data_cov, reg=0.01,
                         noise_cov=noise_cov)
-    stc_fixed = apply_lcmv(evoked, filters, max_ori_out='signed')
+    stc_fixed = apply_lcmv(evoked, filters)
     assert_array_almost_equal(stc_avg, stc_fixed.data)
 
     # use a label so we have few source vertices and delayed computation is
     # not used
     filters = make_lcmv(epochs.info, forward_fixed, data_cov, reg=0.01,
                         noise_cov=noise_cov, label=label)
-    stcs_label = apply_lcmv_epochs(epochs, filters, max_ori_out='signed')
+    stcs_label = apply_lcmv_epochs(epochs, filters)
 
     assert_array_almost_equal(stcs_label[0].data, stcs[0].in_label(label).data)
 
@@ -432,8 +437,13 @@ def test_make_lcmv(tmpdir, reg, proj):
 
 @testing.requires_testing_data
 @pytest.mark.slowtest
-@pytest.mark.parametrize('weight_norm', (None, 'unit-noise-gain', 'nai'))
-@pytest.mark.parametrize('pick_ori', (None, 'max-power', 'vector'))
+@pytest.mark.parametrize('weight_norm, pick_ori', [
+    ('unit-noise-gain', 'max-power'),
+    ('unit-noise-gain', 'vector'),
+    ('unit-noise-gain', None),
+    ('nai', 'vector'),
+    (None, 'max-power'),
+])
 def test_make_lcmv_sphere(pick_ori, weight_norm):
     """Test LCMV with sphere head model."""
     # unit-noise gain beamformer and orientation
@@ -448,16 +458,17 @@ def test_make_lcmv_sphere(pick_ori, weight_norm):
 
     # Test that we get an error if not reducing rank
     with pytest.raises(ValueError, match='Singular matrix detected'):
-        make_lcmv(
-            evoked.info, fwd_sphere, data_cov, reg=0.1,
-            noise_cov=noise_cov, weight_norm=weight_norm,
-            pick_ori=pick_ori, reduce_rank=False, rank='full')
+        with pytest.warns(RuntimeWarning, match='positive semidefinite'):
+            make_lcmv(
+                evoked.info, fwd_sphere, data_cov, reg=0.1,
+                noise_cov=noise_cov, weight_norm=weight_norm,
+                pick_ori=pick_ori, reduce_rank=False, rank='full')
 
     # Now let's reduce it
     filters = make_lcmv(evoked.info, fwd_sphere, data_cov, reg=0.1,
                         noise_cov=noise_cov, weight_norm=weight_norm,
                         pick_ori=pick_ori, reduce_rank=True)
-    stc_sphere = apply_lcmv(evoked, filters, max_ori_out='signed')
+    stc_sphere = apply_lcmv(evoked, filters)
     if isinstance(stc_sphere, VolVectorSourceEstimate):
         stc_sphere = stc_sphere.magnitude()
     else:
@@ -477,33 +488,6 @@ def test_make_lcmv_sphere(pick_ori, weight_norm):
     assert min_ < np.max(max_stc) < max_, (min_, np.max(max_stc), max_)
 
 
-@testing.requires_testing_data
-def test_lcmv_raw():
-    """Test LCMV with raw data."""
-    raw, _, _, _, noise_cov, label, forward, _, _, _ =\
-        _get_data(all_forward=False, epochs=False, data_cov=False)
-
-    tmin, tmax = 0, 20
-    start, stop = raw.time_as_index([tmin, tmax])
-
-    # use only the left-temporal MEG channels for LCMV
-    data_cov = mne.compute_raw_covariance(raw, tmin=tmin, tmax=tmax)
-    filters = make_lcmv(raw.info, forward, data_cov, reg=0.01,
-                        noise_cov=noise_cov, label=label)
-    stc = apply_lcmv_raw(raw, filters, start=start, stop=stop,
-                         max_ori_out='signed')
-
-    assert_array_almost_equal(np.array([tmin, tmax]),
-                              np.array([stc.times[0], stc.times[-1]]),
-                              decimal=2)
-
-    # make sure we get an stc with vertices only in the lh
-    vertno = [forward['src'][0]['vertno'], forward['src'][1]['vertno']]
-    assert len(stc.vertices[0]) == len(np.intersect1d(vertno[0],
-                                                      label.vertices))
-    assert len(stc.vertices[1]) == 0
-
-
 @testing.requires_testing_data
 @pytest.mark.parametrize('weight_norm', (None, 'unit-noise-gain'))
 @pytest.mark.parametrize('pick_ori', ('max-power', 'normal'))
@@ -533,194 +517,20 @@ def test_lcmv_cov(weight_norm, pick_ori):
         assert_allclose(data, stc.data, rtol=1e-12)
 
 
-@testing.requires_testing_data
-def test_lcmv_source_power():
-    """Test LCMV source power computation."""
-    raw, epochs, evoked, data_cov, noise_cov, label, forward,\
-        forward_surf_ori, forward_fixed, forward_vol = _get_data()
-
-    stc_source_power = _lcmv_source_power(epochs.info, forward, noise_cov,
-                                          data_cov, label=label,
-                                          weight_norm='unit-noise-gain')
-
-    max_source_idx = np.argmax(stc_source_power.data)
-    max_source_power = np.max(stc_source_power.data)
-
-    assert max_source_idx == 0, max_source_idx
-    assert 0.4 < max_source_power < 2.4, max_source_power
-
-    # Test picking normal orientation and using a list of CSD matrices
-    stc_normal = _lcmv_source_power(
-        epochs.info, forward_surf_ori, noise_cov, data_cov,
-        pick_ori="normal", label=label, weight_norm='unit-noise-gain')
-
-    # The normal orientation results should always be smaller than free
-    # orientation results
-    assert (np.abs(stc_normal.data[:, 0]) <= stc_source_power.data[:, 0]).all()
-
-    # Test if fixed forward operator is detected when picking normal
-    # orientation
-    pytest.raises(ValueError, _lcmv_source_power, raw.info, forward_fixed,
-                  noise_cov, data_cov, pick_ori="normal")
-
-    # Test if non-surface oriented forward operator is detected when picking
-    # normal orientation
-    pytest.raises(ValueError, _lcmv_source_power, raw.info, forward, noise_cov,
-                  data_cov, pick_ori="normal")
-
-    # Test if volume forward operator is detected when picking normal
-    # orientation
-    pytest.raises(ValueError, _lcmv_source_power, epochs.info, forward_vol,
-                  noise_cov, data_cov, pick_ori="normal")
-
-
-@testing.requires_testing_data
-@pytest.mark.filterwarnings('ignore:.*tf_lcmv is dep.*:DeprecationWarning')
-def test_tf_lcmv():
-    """Test TF beamforming based on LCMV."""
-    label = mne.read_label(fname_label)
-    events = mne.read_events(fname_event)
-    raw = mne.io.read_raw_fif(fname_raw, preload=True)
-    forward = mne.read_forward_solution(fname_fwd)
-
-    event_id, tmin, tmax = 1, -0.2, 0.2
-
-    # Setup for reading the raw data
-    raw.info['bads'] = ['MEG 2443', 'EEG 053']  # 2 bads channels
-
-    # Set up pick list: MEG - bad channels
-    left_temporal_channels = mne.read_selection('Left-temporal')
-    picks = mne.pick_types(raw.info, meg=True,
-                           selection=left_temporal_channels)
-    picks = picks[::2]  # decimate for speed
-    raw.pick_channels([raw.ch_names[ii] for ii in picks])
-    raw.info.normalize_proj()  # avoid projection warnings
-    del picks
-
-    # Read epochs
-    epochs = mne.Epochs(raw, events, event_id, tmin, tmax, proj=True,
-                        baseline=None, preload=False, reject=reject)
-    epochs.load_data()
-
-    freq_bins = [(4, 12), (15, 40)]
-    time_windows = [(-0.1, 0.1), (0.0, 0.2)]
-    win_lengths = [0.2, 0.2]
-    tstep = 0.1
-    reg = 0.05
-
-    source_power = []
-    noise_covs = []
-    for (l_freq, h_freq), win_length in zip(freq_bins, win_lengths):
-        raw_band = raw.copy()
-        raw_band.filter(l_freq, h_freq, method='iir', n_jobs=1,
-                        iir_params=dict(output='ba'))
-        epochs_band = mne.Epochs(
-            raw_band, epochs.events, epochs.event_id, tmin=tmin, tmax=tmax,
-            baseline=None, proj=True)
-        noise_cov = mne.compute_covariance(
-            epochs_band, tmin=tmin, tmax=tmin + win_length)
-        noise_cov = mne.cov.regularize(
-            noise_cov, epochs_band.info, mag=reg, grad=reg, eeg=reg,
-            proj=True, rank=None)
-        noise_covs.append(noise_cov)
-        del raw_band  # to save memory
-
-        # Manually calculating source power in on frequency band and several
-        # time windows to compare to tf_lcmv results and test overlapping
-        if (l_freq, h_freq) == freq_bins[0]:
-            for time_window in time_windows:
-                data_cov = mne.compute_covariance(
-                    epochs_band, tmin=time_window[0], tmax=time_window[1])
-                stc_source_power = _lcmv_source_power(
-                    epochs.info, forward, noise_cov, data_cov,
-                    reg=reg, label=label, weight_norm='unit-noise-gain')
-                source_power.append(stc_source_power.data)
-
-    pytest.raises(ValueError, tf_lcmv, epochs, forward, noise_covs, tmin, tmax,
-                  tstep, win_lengths, freq_bins, reg=reg, label=label)
-    stcs = tf_lcmv(epochs, forward, noise_covs, tmin, tmax, tstep,
-                   win_lengths, freq_bins, reg=reg, label=label, raw=raw)
-
-    assert (len(stcs) == len(freq_bins))
-    assert (stcs[0].shape[1] == 4)
-
-    # Averaging all time windows that overlap the time period 0 to 100 ms
-    source_power = np.mean(source_power, axis=0)
-
-    # Selecting the first frequency bin in tf_lcmv results
-    stc = stcs[0]
-
-    # Comparing tf_lcmv results with _lcmv_source_power results
-    assert_array_almost_equal(stc.data[:, 2], source_power[:, 0])
-
-    # Test if using unsupported max-power orientation is detected
-    pytest.raises(ValueError, tf_lcmv, epochs, forward, noise_covs, tmin, tmax,
-                  tstep, win_lengths, freq_bins=freq_bins,
-                  pick_ori='max-power')
-
-    # Test if incorrect number of noise CSDs is detected
-    # Test if incorrect number of noise covariances is detected
-    pytest.raises(ValueError, tf_lcmv, epochs, forward, [noise_covs[0]], tmin,
-                  tmax, tstep, win_lengths, freq_bins)
-
-    # Test if freq_bins and win_lengths incompatibility is detected
-    pytest.raises(ValueError, tf_lcmv, epochs, forward, noise_covs, tmin, tmax,
-                  tstep, win_lengths=[0, 1, 2], freq_bins=freq_bins)
-
-    # Test if time step exceeding window lengths is detected
-    pytest.raises(ValueError, tf_lcmv, epochs, forward, noise_covs, tmin, tmax,
-                  tstep=0.15, win_lengths=[0.2, 0.1], freq_bins=freq_bins)
-
-    # Test if missing of noise covariance matrix is detected when more than
-    # one channel type is present in the data
-    pytest.raises(ValueError, tf_lcmv, epochs, forward, noise_covs=None,
-                  tmin=tmin, tmax=tmax, tstep=tstep, win_lengths=win_lengths,
-                  freq_bins=freq_bins)
-
-    # Test if unsupported weight normalization specification is detected
-    pytest.raises(ValueError, tf_lcmv, epochs, forward, noise_covs, tmin, tmax,
-                  tstep, win_lengths, freq_bins, weight_norm='nai')
-
-    # Test unsupported pick_ori (vector not supported here)
-    with pytest.raises(ValueError, match='pick_ori'):
-        tf_lcmv(epochs, forward, noise_covs, tmin, tmax, tstep, win_lengths,
-                freq_bins, pick_ori='vector')
-    # Test correct detection of preloaded epochs objects that do not contain
-    # the underlying raw object
-    epochs_preloaded = mne.Epochs(raw, events, event_id, tmin, tmax, proj=True,
-                                  baseline=(None, 0), preload=True)
-    epochs_preloaded._raw = None
-    pytest.raises(ValueError, tf_lcmv, epochs_preloaded, forward,
-                  noise_covs, tmin, tmax, tstep, win_lengths, freq_bins)
-
-    # Pass only one epoch to test if subtracting evoked
-    # responses yields zeros
-    with pytest.warns(RuntimeWarning,
-                      match='Too few samples .* estimate may be unreliable'):
-        stcs = tf_lcmv(epochs[0], forward, noise_covs, tmin, tmax, tstep,
-                       win_lengths, freq_bins, subtract_evoked=True, reg=reg,
-                       label=label, raw=raw)
-
-    assert_array_almost_equal(stcs[0].data, np.zeros_like(stcs[0].data))
-
-
 @pytest.mark.slowtest
 @testing.requires_testing_data
 def test_lcmv_ctf_comp():
     """Test interpolation with compensated CTF data."""
-    ctf_dir = op.join(testing.data_path(download=False), 'CTF')
-    raw_fname = op.join(ctf_dir, 'somMDYO-18av.ds')
-    raw = mne.io.read_raw_ctf(raw_fname, preload=True)
+    raw = mne.io.read_raw_ctf(ctf_fname, preload=True)
+    raw.pick(raw.ch_names[:70])
 
     events = mne.make_fixed_length_events(raw, duration=0.2)[:2]
     epochs = mne.Epochs(raw, events, tmin=-0.1, tmax=0.2)
     evoked = epochs.average()
 
-    with pytest.warns(RuntimeWarning,
-                      match='Too few samples .* estimate may be unreliable'):
-        data_cov = mne.compute_covariance(epochs)
+    data_cov = mne.compute_covariance(epochs)
     fwd = mne.make_forward_solution(evoked.info, None,
-                                    mne.setup_volume_source_space(pos=15.0),
+                                    mne.setup_volume_source_space(pos=30.0),
                                     mne.make_sphere_model())
     with pytest.raises(ValueError, match='reduce_rank'):
         make_lcmv(evoked.info, fwd, data_cov)
@@ -734,9 +544,14 @@ def test_lcmv_ctf_comp():
         make_lcmv(info_comp, fwd, data_cov)
 
 
+@pytest.mark.slowtest
 @testing.requires_testing_data
-@pytest.mark.parametrize('proj', [False, True])
-@pytest.mark.parametrize('weight_norm', (None, 'nai', 'unit-noise-gain'))
+@pytest.mark.parametrize('proj, weight_norm', [
+    (True, 'unit-noise-gain'),
+    (False, 'unit-noise-gain'),
+    (True, None),
+    (True, 'nai'),
+])
 def test_lcmv_reg_proj(proj, weight_norm):
     """Test LCMV with and without proj."""
     raw = mne.io.read_raw_fif(fname_raw, preload=True)
@@ -820,7 +635,7 @@ def test_lcmv_reg_proj(proj, weight_norm):
     (0.05, 'unit-noise-gain', False, None, 83, 86),
     (0.05, 'unit-noise-gain', False, 0.8, 83, 86),  # depth same for wn != None
     # no reg
-    (0.00, 'unit-noise-gain', True, None, 45, 99),  # TODO: Still not stable
+    (0.00, 'unit-noise-gain', True, None, 35, 99),  # TODO: Still not stable
 ])
 def test_localization_bias_fixed(bias_params_fixed, reg, weight_norm, use_cov,
                                  depth, lower, upper):
@@ -839,50 +654,120 @@ def test_localization_bias_fixed(bias_params_fixed, reg, weight_norm, use_cov,
     assert lower <= perc <= upper
 
 
+# Changes here should be synced with test_dics.py
 @pytest.mark.parametrize(
-    'reg, pick_ori, weight_norm, use_cov, depth, lower, upper', [
-        (0.05, 'vector', 'unit-noise-gain-invariant', False, None, 26, 28),
-        (0.05, 'vector', 'unit-noise-gain-invariant', True, None, 40, 42),
-        (0.05, 'vector', 'unit-noise-gain', False, None, 13, 14),
-        (0.05, 'vector', 'unit-noise-gain', True, None, 35, 37),
-        (0.05, 'vector', 'nai', True, None, 35, 37),
-        (0.05, 'vector', None, True, None, 12, 14),
-        (0.05, 'vector', None, True, 0.8, 39, 43),
-        (0.05, 'max-power', 'unit-noise-gain-invariant', False, None, 17, 20),
-        (0.05, 'max-power', 'unit-noise-gain', False, None, 17, 20),
-        (0.05, 'max-power', 'nai', True, None, 21, 24),
-        (0.05, 'max-power', None, True, None, 7, 10),
-        (0.05, 'max-power', None, True, 0.8, 15, 18),
-        (0.05, None, None, True, 0.8, 40, 42),
+    'reg, pick_ori, weight_norm, use_cov, depth, lower, upper, '
+    'lower_ori, upper_ori', [
+        (0.05, 'vector', 'unit-noise-gain-invariant', False, None, 26, 28, 0.82, 0.84),  # noqa: E501
+        (0.05, 'vector', 'unit-noise-gain-invariant', True, None, 40, 42, 0.96, 0.98),  # noqa: E501
+        (0.05, 'vector', 'unit-noise-gain', False, None, 13, 14, 0.79, 0.81),
+        (0.05, 'vector', 'unit-noise-gain', True, None, 35, 37, 0.98, 0.99),
+        (0.05, 'vector', 'nai', True, None, 35, 37, 0.98, 0.99),
+        (0.05, 'vector', None, True, None, 12, 14, 0.97, 0.98),
+        (0.05, 'vector', None, True, 0.8, 39, 43, 0.97, 0.98),
+        (0.05, 'max-power', 'unit-noise-gain-invariant', False, None, 17, 20, 0, 0),  # noqa: E501
+        (0.05, 'max-power', 'unit-noise-gain', False, None, 17, 20, 0, 0),
+        (0.05, 'max-power', 'nai', True, None, 21, 24, 0, 0),
+        (0.05, 'max-power', None, True, None, 7, 10, 0, 0),
+        (0.05, 'max-power', None, True, 0.8, 15, 18, 0, 0),
+        (0.05, None, None, True, 0.8, 40, 42, 0, 0),
         # no reg
-        (0.00, 'vector', None, True, None, 21, 32),
-        (0.00, 'vector', 'unit-noise-gain-invariant', True, None, 50, 65),
-        (0.00, 'vector', 'unit-noise-gain', True, None, 42, 65),
-        (0.00, 'vector', 'nai', True, None, 42, 65),
-        (0.00, 'max-power', None, True, None, 15, 19),
-        (0.00, 'max-power', 'unit-noise-gain-invariant', True, None, 43, 50),
-        (0.00, 'max-power', 'unit-noise-gain', True, None, 43, 50),
-        (0.00, 'max-power', 'nai', True, None, 43, 50),
+        (0.00, 'vector', None, True, None, 23, 24, 0.96, 0.97),
+        (0.00, 'vector', 'unit-noise-gain-invariant', True, None, 52, 54, 0.95, 0.96),  # noqa: E501
+        (0.00, 'vector', 'unit-noise-gain', True, None, 44, 48, 0.97, 0.99),
+        (0.00, 'vector', 'nai', True, None, 44, 48, 0.97, 0.99),
+        (0.00, 'max-power', None, True, None, 14, 15, 0, 0),
+        (0.00, 'max-power', 'unit-noise-gain-invariant', True, None, 35, 37, 0, 0),  # noqa: E501
+        (0.00, 'max-power', 'unit-noise-gain', True, None, 35, 37, 0, 0),
+        (0.00, 'max-power', 'nai', True, None, 35, 37, 0, 0),
     ])
 def test_localization_bias_free(bias_params_free, reg, pick_ori, weight_norm,
-                                use_cov, depth, lower, upper):
+                                use_cov, depth, lower, upper,
+                                lower_ori, upper_ori):
     """Test localization bias for free-orientation LCMV."""
     evoked, fwd, noise_cov, data_cov, want = bias_params_free
     if not use_cov:
         evoked.pick_types(meg='grad')
         noise_cov = None
-    loc = apply_lcmv(evoked, make_lcmv(evoked.info, fwd, data_cov, reg,
-                                       noise_cov, pick_ori=pick_ori,
-                                       weight_norm=weight_norm,
-                                       depth=depth)).data
+    with _record_warnings():  # rank deficiency of data_cov
+        filters = make_lcmv(evoked.info, fwd, data_cov, reg,
+                            noise_cov, pick_ori=pick_ori,
+                            weight_norm=weight_norm,
+                            depth=depth)
+    loc = apply_lcmv(evoked, filters).data
+    if pick_ori == 'vector':
+        ori = loc.copy() / np.linalg.norm(loc, axis=1, keepdims=True)
+    else:
+        # doesn't make sense for pooled (None) or max-power (can't be all 3)
+        ori = None
     loc = np.linalg.norm(loc, axis=1) if pick_ori == 'vector' else np.abs(loc)
     # Compute the percentage of sources for which there is no loc bias:
-    perc = (want == np.argmax(loc, axis=0)).mean() * 100
+    max_idx = np.argmax(loc, axis=0)
+    perc = (want == max_idx).mean() * 100
     assert lower <= perc <= upper
+    _assert_free_ori_match(ori, max_idx, lower_ori, upper_ori)
 
 
-@pytest.mark.parametrize('weight_norm', ('nai', 'unit-noise-gain'))
-@pytest.mark.parametrize('pick_ori', ('vector', 'max-power', None))
+# Changes here should be synced with the ones above, but these have meaningful
+# orientation values
+@pytest.mark.parametrize(
+    'reg, weight_norm, use_cov, depth, lower, upper, lower_ori, upper_ori', [
+        (0.05, 'unit-noise-gain-invariant', False, None, 38, 40, 0.54, 0.55),
+        (0.05, 'unit-noise-gain', False, None, 38, 40, 0.54, 0.55),
+        (0.05, 'nai', True, None, 56, 57, 0.59, 0.61),
+        (0.05, None, True, None, 27, 28, 0.56, 0.57),
+        (0.05, None, True, 0.8, 42, 43, 0.56, 0.57),
+        # no reg
+        (0.00, None, True, None, 50, 51, 0.58, 0.59),
+        (0.00, 'unit-noise-gain-invariant', True, None, 73, 75, 0.59, 0.61),
+        (0.00, 'unit-noise-gain', True, None, 73, 75, 0.59, 0.61),
+        (0.00, 'nai', True, None, 73, 75, 0.59, 0.61),
+    ])
+def test_orientation_max_power(bias_params_fixed, bias_params_free,
+                               reg, weight_norm, use_cov, depth, lower, upper,
+                               lower_ori, upper_ori):
+    """Test orientation selection for bias for max-power LCMV."""
+    # we simulate data for the fixed orientation forward and beamform using
+    # the free orientation forward, and check the orientation match at the end
+    evoked, _, noise_cov, data_cov, want = bias_params_fixed
+    fwd = bias_params_free[1]
+    if not use_cov:
+        evoked.pick_types(meg='grad')
+        noise_cov = None
+    filters = make_lcmv(evoked.info, fwd, data_cov, reg,
+                        noise_cov, pick_ori='max-power',
+                        weight_norm=weight_norm,
+                        depth=depth)
+    loc = apply_lcmv(evoked, filters).data
+    ori = filters['max_power_ori']
+    assert ori.shape == (246, 3)
+    loc = np.abs(loc)
+    # Compute the percentage of sources for which there is no loc bias:
+    max_idx = np.argmax(loc, axis=0)
+    mask = want == max_idx  # ones that localized properly
+    perc = mask.mean() * 100
+    assert lower <= perc <= upper
+    # Compute the dot products of our forward normals and
+    assert fwd['coord_frame'] == FIFF.FIFFV_COORD_HEAD
+    nn = np.concatenate(
+        [s['nn'][v] for s, v in zip(fwd['src'], filters['vertices'])])
+    nn = nn[want]
+    nn = apply_trans(invert_transform(fwd['mri_head_t']), nn, move=False)
+    assert_allclose(np.linalg.norm(nn, axis=1), 1, atol=1e-6)
+    assert_allclose(np.linalg.norm(ori, axis=1), 1, atol=1e-12)
+    dots = np.abs((nn[mask] * ori[mask]).sum(-1))
+    assert_array_less(dots, 1)
+    assert_array_less(0, dots)
+    got = np.mean(dots)
+    assert lower_ori < got < upper_ori
+
+
+@pytest.mark.parametrize('weight_norm, pick_ori', [
+    pytest.param('nai', 'max-power', marks=pytest.mark.slowtest),
+    ('unit-noise-gain', 'vector'),
+    ('unit-noise-gain', 'max-power'),
+    pytest.param('unit-noise-gain', None, marks=pytest.mark.slowtest),
+])
 def test_depth_does_not_matter(bias_params_free, weight_norm, pick_ori):
     """Test that depth weighting does not matter for normalized filters."""
     evoked, fwd, noise_cov, data_cov, _ = bias_params_free
@@ -918,12 +803,24 @@ def test_lcmv_maxfiltered():
         make_lcmv(epochs.info, fwd, data_cov, rank=use_rank)
 
 
+# To reduce test time, only test combinations that should matter rather than
+# all of them
 @testing.requires_testing_data
-@pytest.mark.parametrize('pick_ori', ['vector', 'max-power', 'normal'])
-@pytest.mark.parametrize(
-    'weight_norm', ['unit-noise-gain', 'nai', 'unit-noise-gain-invariant'])
-@pytest.mark.parametrize('reg', (0.05, 0.))
-@pytest.mark.parametrize('inversion', ['matrix', 'single'])
+@pytest.mark.parametrize('pick_ori, weight_norm, reg, inversion', [
+    ('vector', 'unit-noise-gain-invariant', 0.05, 'matrix'),
+    ('vector', 'unit-noise-gain-invariant', 0.05, 'single'),
+    ('vector', 'unit-noise-gain', 0.05, 'matrix'),
+    ('vector', 'unit-noise-gain', 0.05, 'single'),
+    ('vector', 'unit-noise-gain', 0.0, 'matrix'),
+    ('vector', 'unit-noise-gain', 0.0, 'single'),
+    ('vector', 'nai', 0.05, 'matrix'),
+    ('max-power', 'unit-noise-gain', 0.05, 'matrix'),
+    ('max-power', 'unit-noise-gain', 0.0, 'single'),
+    ('max-power', 'unit-noise-gain', 0.05, 'single'),
+    ('max-power', 'unit-noise-gain-invariant', 0.05, 'matrix'),
+    ('normal', 'unit-noise-gain', 0.05, 'matrix'),
+    ('normal', 'nai', 0.0, 'matrix'),
+])
 def test_unit_noise_gain_formula(pick_ori, weight_norm, reg, inversion):
     """Test unit-noise-gain filter against formula."""
     raw = mne.io.read_raw_fif(fname_raw, preload=True)
@@ -1028,5 +925,4 @@ def test_api():
     dics_names[dics_names.index('csd')] = 'data_cov'
     dics_names[dics_names.index('noise_csd')] = 'noise_cov'
     dics_names.pop(dics_names.index('real_filter'))  # not a thing for LCMV
-    dics_names.pop(dics_names.index('normalize_fwd'))  # deprecated
     assert lcmv_names == dics_names
